import { __extends, __makeTemplateObject } from "tslib";
import React from 'react';
import { Link } from 'react-router';
import styled from '@emotion/styled';
import moment from 'moment-timezone';
import DateTime from 'app/components/dateTime';
import ErrorBoundary from 'app/components/errorBoundary';
import FeatureBadge from 'app/components/featureBadge';
import FileSize from 'app/components/fileSize';
import ExternalLink from 'app/components/links/externalLink';
import NavigationButtonGroup from 'app/components/navigationButtonGroup';
import Placeholder from 'app/components/placeholder';
import QuickTrace from 'app/components/quickTrace';
import { generateTraceTarget } from 'app/components/quickTrace/utils';
import Tooltip from 'app/components/tooltip';
import { IconWarning } from 'app/icons';
import { t } from 'app/locale';
import ConfigStore from 'app/stores/configStore';
import space from 'app/styles/space';
import { trackAnalyticsEvent } from 'app/utils/analytics';
import getDynamicText from 'app/utils/getDynamicText';
import QuickTraceQuery from 'app/utils/performance/quickTrace/quickTraceQuery';
var formatDateDelta = function (reference, observed) {
    var duration = moment.duration(Math.abs(+observed - +reference));
    var hours = Math.floor(+duration / (60 * 60 * 1000));
    var minutes = duration.minutes();
    var results = [];
    if (hours) {
        results.push(hours + " hour" + (hours !== 1 ? 's' : ''));
    }
    if (minutes) {
        results.push(minutes + " minute" + (minutes !== 1 ? 's' : ''));
    }
    if (results.length === 0) {
        results.push('a few seconds');
    }
    return results.join(', ');
};
var GroupEventToolbar = /** @class */ (function (_super) {
    __extends(GroupEventToolbar, _super);
    function GroupEventToolbar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GroupEventToolbar.prototype.shouldComponentUpdate = function (nextProps) {
        return this.props.event.id !== nextProps.event.id;
    };
    GroupEventToolbar.prototype.handleTraceLink = function (organization) {
        trackAnalyticsEvent({
            eventKey: 'quick_trace.trace_id.clicked',
            eventName: 'Quick Trace: Trace ID clicked',
            organization_id: parseInt(organization.id, 10),
            source: 'issues',
        });
    };
    GroupEventToolbar.prototype.renderQuickTrace = function () {
        var _this = this;
        var _a = this.props, event = _a.event, organization = _a.organization, location = _a.location;
        return (<ErrorBoundary mini>
        <QuickTraceQuery event={event} location={location} orgSlug={organization.slug}>
          {function (results) { return (<React.Fragment>
              {!results.isLoading && results.error === null && results.trace !== null && (<LinkContainer>
                  <Link to={generateTraceTarget(event, organization)} onClick={function () { return _this.handleTraceLink(organization); }}>
                    View Full Trace
                    <FeatureBadge type="beta"/>
                  </Link>
                </LinkContainer>)}
              <QuickTraceWrapper>
                {results.isLoading ? (<Placeholder height="24px"/>) : results.error || results.trace === null ? null : (<QuickTrace event={event} quickTrace={results} location={location} organization={organization} anchor="left" errorDest="issue" transactionDest="performance"/>)}
              </QuickTraceWrapper>
            </React.Fragment>); }}
        </QuickTraceQuery>
      </ErrorBoundary>);
    };
    GroupEventToolbar.prototype.getDateTooltip = function () {
        var _a;
        var evt = this.props.event;
        var user = ConfigStore.get('user');
        var options = (_a = user === null || user === void 0 ? void 0 : user.options) !== null && _a !== void 0 ? _a : {};
        var format = options.clock24Hours ? 'HH:mm:ss z' : 'LTS z';
        var dateCreated = moment(evt.dateCreated);
        var dateReceived = evt.dateReceived ? moment(evt.dateReceived) : null;
        return (<DescriptionList className="flat">
        <dt>Occurred</dt>
        <dd>
          {dateCreated.format('ll')}
          <br />
          {dateCreated.format(format)}
        </dd>
        {dateReceived && (<React.Fragment>
            <dt>Received</dt>
            <dd>
              {dateReceived.format('ll')}
              <br />
              {dateReceived.format(format)}
            </dd>
            <dt>Latency</dt>
            <dd>{formatDateDelta(dateCreated, dateReceived)}</dd>
          </React.Fragment>)}
      </DescriptionList>);
    };
    GroupEventToolbar.prototype.render = function () {
        var _a, _b;
        var evt = this.props.event;
        var _c = this.props, organization = _c.organization, location = _c.location;
        var groupId = this.props.group.id;
        var baseEventsPath = "/organizations/" + organization.slug + "/issues/" + groupId + "/events/";
        // TODO: possible to define this as a route in react-router, but without a corresponding
        //       React component?
        var jsonUrl = "/organizations/" + organization.slug + "/issues/" + groupId + "/events/" + evt.id + "/json/";
        var latencyThreshold = 30 * 60 * 1000; // 30 minutes
        var isOverLatencyThreshold = evt.dateReceived &&
            Math.abs(+moment(evt.dateReceived) - +moment(evt.dateCreated)) > latencyThreshold;
        var hasQuickTraceView = (organization.features.includes('trace-view-summary') ||
            organization.features.includes('trace-view-quick')) &&
            ((_b = (_a = evt.contexts) === null || _a === void 0 ? void 0 : _a.trace) === null || _b === void 0 ? void 0 : _b.trace_id);
        return (<Wrapper>
        <StyledNavigationButtonGroup location={location} hasPrevious={!!evt.previousEventID} hasNext={!!evt.nextEventID} urls={[
                baseEventsPath + "oldest/",
                "" + baseEventsPath + evt.previousEventID + "/",
                "" + baseEventsPath + evt.nextEventID + "/",
                baseEventsPath + "latest/",
            ]}/>
        <Heading>
          {t('Event')}{' '}
          <EventIdLink to={"" + baseEventsPath + evt.id + "/"}>{evt.eventID}</EventIdLink>
          <LinkContainer>
            <ExternalLink href={jsonUrl}>
              {'JSON'} (<FileSize bytes={evt.size}/>)
            </ExternalLink>
          </LinkContainer>
        </Heading>
        <Tooltip title={this.getDateTooltip()} disableForVisualTest>
          <StyledDateTime date={getDynamicText({ value: evt.dateCreated, fixed: 'Dummy timestamp' })}/>
          {isOverLatencyThreshold && <StyledIconWarning color="yellow300"/>}
        </Tooltip>
        {hasQuickTraceView && this.renderQuickTrace()}
      </Wrapper>);
    };
    return GroupEventToolbar;
}(React.Component));
var Wrapper = styled('div')(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  position: relative;\n  margin-bottom: -5px;\n  /* z-index seems unnecessary, but increasing (instead of removing) just in case(billy) */\n  /* Fixes tooltips in toolbar having lower z-index than .btn-group .btn.active */\n  z-index: 3;\n  padding: 20px 30px 20px 40px;\n\n  @media (max-width: 767px) {\n    display: none;\n  }\n"], ["\n  position: relative;\n  margin-bottom: -5px;\n  /* z-index seems unnecessary, but increasing (instead of removing) just in case(billy) */\n  /* Fixes tooltips in toolbar having lower z-index than .btn-group .btn.active */\n  z-index: 3;\n  padding: 20px 30px 20px 40px;\n\n  @media (max-width: 767px) {\n    display: none;\n  }\n"])));
var EventIdLink = styled(Link)(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n  font-weight: normal;\n"], ["\n  font-weight: normal;\n"])));
var Heading = styled('h4')(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n  line-height: 1.3;\n  margin: 0;\n  font-size: ", ";\n"], ["\n  line-height: 1.3;\n  margin: 0;\n  font-size: ", ";\n"])), function (p) { return p.theme.fontSizeLarge; });
var StyledNavigationButtonGroup = styled(NavigationButtonGroup)(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n  float: right;\n"], ["\n  float: right;\n"])));
var StyledIconWarning = styled(IconWarning)(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n  margin-left: ", ";\n  position: relative;\n  top: ", ";\n"], ["\n  margin-left: ", ";\n  position: relative;\n  top: ", ";\n"])), space(0.5), space(0.25));
var StyledDateTime = styled(DateTime)(templateObject_6 || (templateObject_6 = __makeTemplateObject(["\n  border-bottom: 1px dotted #dfe3ea;\n  color: ", ";\n"], ["\n  border-bottom: 1px dotted #dfe3ea;\n  color: ", ";\n"])), function (p) { return p.theme.subText; });
var LinkContainer = styled('span')(templateObject_7 || (templateObject_7 = __makeTemplateObject(["\n  margin-left: ", ";\n  padding-left: ", ";\n  position: relative;\n  font-weight: normal;\n\n  &:before {\n    display: block;\n    position: absolute;\n    content: '';\n    left: 0;\n    top: 2px;\n    height: 14px;\n    border-left: 1px solid ", ";\n  }\n"], ["\n  margin-left: ", ";\n  padding-left: ", ";\n  position: relative;\n  font-weight: normal;\n\n  &:before {\n    display: block;\n    position: absolute;\n    content: '';\n    left: 0;\n    top: 2px;\n    height: 14px;\n    border-left: 1px solid ", ";\n  }\n"])), space(1), space(1), function (p) { return p.theme.border; });
var DescriptionList = styled('dl')(templateObject_8 || (templateObject_8 = __makeTemplateObject(["\n  text-align: left;\n  margin: 0;\n  min-width: 200px;\n  max-width: 250px;\n"], ["\n  text-align: left;\n  margin: 0;\n  min-width: 200px;\n  max-width: 250px;\n"])));
var QuickTraceWrapper = styled('div')(templateObject_9 || (templateObject_9 = __makeTemplateObject(["\n  margin-top: ", ";\n"], ["\n  margin-top: ", ";\n"])), space(0.5));
export default GroupEventToolbar;
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5, templateObject_6, templateObject_7, templateObject_8, templateObject_9;
//# sourceMappingURL=eventToolbar.jsx.map